import java.util.Arrays;

/* Task1: slicing and merging on 1 thread, sorting slices is parralelized */
public class Task1 {

  /* Create new sorted array by merging 2 smaller sorted arrays */
  private static int[] merge(int[] arr1, int[] arr2) {
    int[] arr3 = new int[arr1.length + arr2.length];
    int idx1 = 0;
    int idx2 = 0;
    int idx3 = 0;
    while (idx1 < arr1.length && idx2 < arr2.length) {
      if (arr1[idx1] < arr2[idx2])
        arr3[idx3++] = arr1[idx1++];
      else
        arr3[idx3++] = arr2[idx2++];
    }
    while(idx1 < arr1.length)
      arr3[idx3++] = arr1[idx1++];
    while(idx2 < arr2.length)
      arr3[idx3++] = arr2[idx2++];
    return arr3;
  }

  /* Creates an array of arrays by slicing a bigger array into smaller chunks */
  private static int[][] slice(int[] arr, int k) {
    int[][] arr2d = new int[k][];
    for (int i = 0; i < k; i++) {
      int from = ((i)   * arr.length) / k;
      int to   = ((i+1) * arr.length) / k;
      arr2d[i] = Arrays.copyOfRange(arr, from, to);
    }
    return arr2d;
  }


  /* Creates a sorted version of any int array */
  public static int[] sort(int[] array) {

    /* Initialize variables */
    int tnum = Runtime.getRuntime().availableProcessors();
    Thread[] threadpool = new Thread[tnum];
    int[] sorted = new int[0];

    /* Turn initial array into array of smaller arrays */
    int[][] arr2d = Task1.slice(array, tnum);

    /* parralelized sort on the smaller arrays */
    for (int i = 0; i < tnum; i++) {
      int[] arr = arr2d[i];
      threadpool[i] = new Thread(() -> Arrays.sort(arr));
      threadpool[i].start();
    }
    try {for (Thread t : threadpool) t.join();}
    catch (InterruptedException ie) {ie.printStackTrace();}

    /* Merge sorted smaller arrays into a singular larger one */
    for(int i = 0; i < arr2d.length; i++)
      sorted = Task1.merge(sorted, arr2d[i]);

    /* Return fully sorted array */
    return sorted;
  }
}
